tfel_enable_cxx_compiler_flag(COMPILER_WARNINGS  "Wall" "W" "Wextra" "pedantic" "Wshadow")
tfel_enable_cxx_compiler_flag(COMPILER_WARNINGS  "Wpointer-arith" "Wcast-qual" "Wcast-align")
tfel_enable_cxx_compiler_flag(COMPILER_WARNINGS  "Wwrite-strings" "Wctor-dtor-privacy" "Wnon-virtual-dtor")
tfel_enable_cxx_compiler_flag(COMPILER_WARNINGS  "Woverloaded-virtual" "Wreturn-type" "Wfloat-equal")
tfel_enable_cxx_compiler_flag(COMPILER_WARNINGS  "Wno-endif-labels" "Wsign-compare" "Wmissing-format-attribute")
tfel_enable_cxx_compiler_flag(COMPILER_WARNINGS  "Wno-multichar" "Wno-deprecated-declarations" "Wpacked")
tfel_enable_cxx_compiler_flag(COMPILER_WARNINGS  "Wredundant-decls" "Wlong-long" "Wdisabled-optimization")
tfel_enable_cxx_compiler_flag(COMPILER_WARNINGS  "Wunknown-pragmas" "Wundef" "Wreorder")

tfel_enable_cxx_compiler_flag(OPTIMISATION_FLAGS_MARCH "march=native")
tfel_enable_cxx_compiler_flag(OPTIMISATION_FLAGS2 "ffast-math")

tfel_enable_cxx_compiler_flag(VISIBILITY_FLAGS "fvisibility=hidden")
tfel_enable_cxx_compiler_flag(VISIBILITY_FLAGS "fvisibility-inlines-hidden")

set(OPTIMISATION_FLAGS "-DNO_RUNTIME_CHECK_BOUNDS ${OPTIMISATION_FLAGS}")

if((NOT CMAKE_BUILD_TYPE) OR (CMAKE_BUILD_TYPE STREQUAL "Release"))
  set(OPTIMISATION_FLAGS "-O2 -DNDEBUG ${OPTIMISATION_FLAGS}")
endif((NOT CMAKE_BUILD_TYPE) OR (CMAKE_BUILD_TYPE STREQUAL "Release"))

if(CMAKE_BUILD_TYPE STREQUAL "Debug")
  add_definitions("-g")
endif(CMAKE_BUILD_TYPE STREQUAL "Debug")

if(HAVE_FORTRAN)
  # we associate clang with the gnu fortran compiler
  include(cmake/modules/gnu-fortran-compiler.cmake)
endif(HAVE_FORTRAN)

option(enable-libcxx "use LLVM C++ Standard library" OFF)
if(enable-libcxx)
  if(CMAKE_BUILD_TYPE STREQUAL "Debug")
    set(CMAKE_CXX_FLAGS_DEBUG     "-stdlib=libc++ ${CMAKE_CXX_FLAGS_DEBUG}")
  elseif(CMAKE_BUILD_TYPE STREQUAL "Coverage")
    set(CMAKE_CXX_FLAGS_COVERAGE  "-stdlib=libc++ ${CMAKE_CXX_FLAGS_COVERAGE}")
  elseif(CMAKE_BUILD_TYPE STREQUAL "Profiling")
    set(CMAKE_CXX_FLAGS_PROFILING "-stdlib=libc++ ${CMAKE_CXX_FLAGS_PROFILING}")
  elseif(CMAKE_BUILD_TYPE STREQUAL "Release")
    set(CMAKE_CXX_FLAGS_RELEASE   "-stdlib=libc++ ${CMAKE_CXX_FLAGS_RELEASE}")
  else(CMAKE_BUILD_TYPE STREQUAL "Debug")
    set(CMAKE_CXX_FLAGS           "-stdlib=libc++ ${CMAKE_CXX_FLAGS}")
  endif(CMAKE_BUILD_TYPE STREQUAL "Debug")
  set(OPTIMISATION_FLAGS "-stdlib=libc++ ${OPTIMISATION_FLAGS}")
endif(enable-libcxx)

if(enable-cxx-11)
  if(${CMAKE_CXX_COMPILER_VERSION} VERSION_LESS 3.4)
    message(FATAL_ERROR "TFEL C++11 support is only available for gcc version >= 3.4")
  endif(${CMAKE_CXX_COMPILER_VERSION} VERSION_LESS 3.4)
  if(CMAKE_BUILD_TYPE STREQUAL "Debug")
    set(CMAKE_CXX_FLAGS_DEBUG     "-std=c++11 ${CMAKE_CXX_FLAGS_DEBUG}")
  elseif(CMAKE_BUILD_TYPE STREQUAL "Coverage")
    set(CMAKE_CXX_FLAGS_COVERAGE  "-std=c++11 ${CMAKE_CXX_FLAGS_COVERAGE}")
  elseif(CMAKE_BUILD_TYPE STREQUAL "Profiling")
    set(CMAKE_CXX_FLAGS_PROFILING "-std=c++11 ${CMAKE_CXX_FLAGS_PROFILING}")
  elseif(CMAKE_BUILD_TYPE STREQUAL "Release")
    set(CMAKE_CXX_FLAGS_RELEASE   "-std=c++11 ${CMAKE_CXX_FLAGS_RELEASE}")
  else(CMAKE_BUILD_TYPE STREQUAL "Debug")
    set(CMAKE_CXX_FLAGS           "-std=c++11 ${CMAKE_CXX_FLAGS}")
  endif(CMAKE_BUILD_TYPE STREQUAL "Debug")
  set(OPTIMISATION_FLAGS "-std=c++11 ${OPTIMISATION_FLAGS}")
endif(enable-cxx-11)
